__title__ = 'Django Pathfinder Statcrunch'
__description__ = 'A reusable Django application using the Krypted Gaming template.'
__package_name__ = 'django-pathfinder-statcrunch'
__github_url__ = 'https://github.com/porowns/django-pathfinder-statcrunch'
__version__ = '0.0.1'
__author__ = 'porowns'
__author_email__ = 'porowns@gmail.com'
__license__ = 'MIT License'
__copyright__ = f"Copyright © 2017-2020 porowns. All rights reserved."
default_app_config = "django_pathfinder_statcrunch.apps.DjangoPathfinderStatcrunchConfig"



# Generated by Django 2.2.13 on 2020-11-02 14:28

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='MonthlyReport',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date', models.DateField(auto_now=True)),
                ('statistics', models.TextField()),
            ],
        ),
        migrations.CreateModel(
            name='PathfinderUpload',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('file', models.FileField(upload_to='')),
                ('report', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_pathfinder_statcrunch.MonthlyReport')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from celery import task, shared_task
from .models import PathfinderUpload
from django_eveonline_connector.models import EveCharacter, PrimaryEveCharacterAssociation
import csv, logging 
logger = logging.getLogger(__name__)

@shared_task
def add_pathfinder_upload_to_report(upload_pk):
    upload = PathfinderUpload.objects.get(pk=upload_pk)
    report = upload.report.get_stats()
    if not report:
        report = {
            'characters': {},
            'corporations': {}
        }
    # parse report
  
    f = open(upload.file.path, 'r')
    for line in f:
        data = line.split(",")
        if len(data) != 57:
            logger.info(f"Skipping line in PF read: {line}")
            continue
        name = (data[2]).replace('"', "").strip()
        try:
            current_character = EveCharacter.objects.get(name=name)
            primary_character = current_character.get_primary_character(current_character.token.user)
            if primary_character:
                current_character = primary_character
        except Exception as e:
            logger.error(e)
            logger.warning(f"Skipping PF line, failed to find character for {name}")
            continue
        signatures = int(data[55].replace('"', "").strip())
        if current_character.name in report:
            report['characters'][current_character.name]['signatures'] += int(signatures)
        else:
            report['characters'][current_character.name] = {
                "signatures": int(signatures),
              "corporation": current_character.corporation.name,
            }

        if current_character.corporation.track_corporation:
            if current_character.corporation.name in report['corporations']:
                report['corporations'][current_character.corporation.name] += int(
                    signatures)
            else:
                report['corporations'][current_character.corporation.name] = int(
                    signatures)

    for key in report['corporations']:
        players = PrimaryEveCharacterAssociation.objects.filter(
            character__corporation__name=key)
        for player in players:
            primary_character = player.character 
            if primary_character.name not in report['characters']:
                report['characters'][primary_character.name] = {
                    'corporation': key,
                    'signatures': -1,
                }

    
        
    # update stats
    upload.report.update_stats(report)
    upload.delete()



from django.db import models
from django.contrib.auth.models import User 
from datetime import datetime
import json

class PathfinderUpload(models.Model):
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    report = models.ForeignKey("MonthlyReport", on_delete=models.CASCADE)
    file = models.FileField()

class MonthlyReport(models.Model):
    date = models.DateField(auto_now=True)
    statistics = models.TextField(blank=True, null=True)

    def __str__(self):
        return self.date.strftime("%B %Y")

    def save(self, *args, **kwargs):
        if MonthlyReport.objects.filter(date__startswith=datetime.now().strftime('%Y-%m-')).exists():
            report = MonthlyReport.objects.get(date__startswith=datetime.now().strftime('%Y-%m-'))
            if report.pk != self.pk:
                raise Exception("Report exists")
        super(MonthlyReport, self).save(*args, **kwargs)

    def get_stats(self):
        if not self.statistics:
            return json.loads("{}")
        return json.loads(self.statistics)

    def update_stats(self, report):
        self.statistics = json.dumps(report)
        self.save()



from django.db import transaction
from django.dispatch import receiver
from django.db.models.signals import post_save
from .models import PathfinderUpload
from .tasks import add_pathfinder_upload_to_report

@receiver(post_save, sender=PathfinderUpload)
def handle_pathfinder_upload(sender, **kwargs):
    def on_commit():
        instance = kwargs.get('instance')
        add_pathfinder_upload_to_report.apply_async(args=[instance.pk])

    transaction.on_commit(on_commit)


from django.contrib.staticfiles.urls import staticfiles_urlpatterns
from django.conf import settings
from django.contrib import admin
from django.urls import path, re_path
from . import views 

# SSO
urlpatterns = [
    path('reports/', views.list_reports,
         name="django-pathfinder-statcrunch-list-reports"),
    path('reports/<int:pk>/', views.view_report,
        name="django-pathfinder-statcrunch-view-report"),
]



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

settings = __import__('django_pathfinder_statcrunch')
setup(
    name=settings.__package_name__,
    version=settings.__version__,
    packages=find_packages(),
    include_package_data=True,
    license=settings.__license__,
    description=settings.__description__,
    long_description=README,
    long_description_content_type='text/markdown',
    url=settings.__github_url__,
    author=settings.__author__,
    author_email=settings.__author_email__,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=[
    ]
)



from django.shortcuts import render
from .models import MonthlyReport
from django_eveonline_connector.models import PrimaryEveCharacterAssociation

# Create your views here.
def list_reports(request):
    return render(request, 'django_pathfinder_statcrunch/adminlte/list_reports.html', context={
        "reports": MonthlyReport.objects.all()
    })

def view_report(request, pk):
    context = {}
    report = MonthlyReport.objects.get(pk=pk)
    context['characters'] = report.get_stats()['characters']
    context['corporations'] = {}
    for key in report.get_stats()['corporations']:
        player_count = PrimaryEveCharacterAssociation.objects.filter(
            character__corporation__name=key).count()
        context['corporations'][key] = {
            'signatures': report.get_stats()['corporations'][key],
            'signatures_per_capita': report.get_stats()['corporations'][key] / player_count,
        }
    return render(request, 'django_pathfinder_statcrunch/adminlte/view_report.html', context=context)



from django.apps import AppConfig


class DjangoPathfinderStatcrunchConfig(AppConfig):
    name = 'django_pathfinder_statcrunch'
    url_slug = 'pathfinder'

    def ready(self):
        from django.db.models.signals import post_save
        from .models import PathfinderUpload
        from .signals import handle_pathfinder_upload

        post_save.connect(handle_pathfinder_upload, sender=PathfinderUpload)